##############################################################################
#
# DavPack -- A set of patches for all things related to WebDAV
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Some portions of this module are copyright Benjamin Saller and respective
# authors.
# The original copyright statement can be found on ArchetypesLicense.txt.
#
##############################################################################
"""
$Id: content.py 609 2004-11-05 19:15:56Z sidnei $
"""

from Products.CMFCore.utils import getToolByName
from ZODB.POSException import ConflictError

from Products.Archetypes.utils import shasattr
from Products.Archetypes.utils import mapply
from Products.Archetypes.BaseContent import BaseContent as OrigBase

_marker = object()

class BaseContent(OrigBase):

    def PUT(self, REQUEST=None, RESPONSE=None):
        """ HTTP PUT handler with marshalling support
        """

        if not REQUEST:
            REQUEST = self.REQUEST
        if not RESPONSE:
            RESPONSE = REQUEST.RESPONSE
        if not self.Schema().hasLayer('marshall'):
            RESPONSE.setStatus(501) # Not implemented
            return RESPONSE

        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)

        file = REQUEST.get('BODYFILE', _marker)
        if file is _marker:
            data = REQUEST.get('BODY', _marker)
            if data is _marker:
                raise AttributeError, 'REQUEST neither has a BODY nor a BODYFILE'
        else:
            data = ''
            file.seek(0)

        mimetype = REQUEST.get_header('content-type', None)

        try:
            filename = REQUEST._steps[-2] #XXX fixme, use a real name
        except ConflictError:
            raise
        except:
            filename = (getattr(file, 'filename', None) or
                        getattr(file, 'name', None))

        # XXX remove after we are using global services
        # use the request to find an object in the traversal hirachy that is
        # able to acquire a mimetypes_registry instance
        # This is a hack to avoid the acquisition problem on FTP/WebDAV object
        # creation
        parents = [self] + REQUEST.get('PARENTS', [])
        context = None
        for parent in parents:
            if getToolByName(parent, 'mimetypes_registry', None) is not None:
                context = parent
                break

        # Marshall the data
        marshaller = self.Schema().getLayerImpl('marshall')

        args = [self, data]
        kwargs = {'file':file,
                  'context':context,
                  'mimetype':mimetype,
                  'filename':filename,
                  'REQUEST':REQUEST,
                  'RESPONSE':RESPONSE}
        ddata = mapply(marshaller.demarshall, *args, **kwargs)

        if (shasattr(self, 'demarshall_hook') and self.demarshall_hook):
            self.demarshall_hook(ddata)

        self.reindexObject()
        RESPONSE.setStatus(204)
        return RESPONSE
